/*
  Copyright (c) 2022  Kazuhiro Ito (kzhr@d1.dion.ne.jp)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

#include "common.h"

static char *unescape_value(char *string) {
  if (string && strlen(string) >= 2 &&
      *string == '\'' && *(string + strlen(string) - 1) == '\'') {
    *(string + strlen(string) - 1) = 0;
    return string + 1;
  }
  return string;
}

int variables_set
(KVS *variables, const unsigned char *key, const unsigned char *value,
 bool system) {
  if (system) {
    unsigned char *new_key = snprintf_buffer("_%s", key);
    if (!new_key) return -1;

    int result = kvs_set(variables, new_key, value);
    free (new_key);
    return result;
  }

  if (key[0] == '_') return -1;

  return kvs_set(variables, key, value);
}

int variables_unset(KVS *variables, const unsigned char *key) {
  if (key[0] == '_') return -1;

  return kvs_unset(variables, key);
}

const unsigned char *variables_get
(KVS *variables, const unsigned char *key, const unsigned char *def) {
  unsigned char *result = kvs_get(variables, key);
  if (!result) {
    unsigned char *new_key = snprintf_buffer("_%s", key);
    if (new_key) {
      result = kvs_get(variables, new_key);
      free(new_key);
    }
  }

  DEBUG("variables %s's value is %s", key, result ? (char *)result : "(not found)");
  return result ? result : def;
}

bool variables_get_bool(KVS *variables, const unsigned char *key, bool def) {
  unsigned char *value = kvs_get(variables, key);
  if (!value) return def;
  if (!strcmp(value, "yes")
      || !strcmp(value, "on")) {
    return true;
  } else if (!strcmp(value, "no")
	     || !strcmp(value, "off")) {
    return false;
  }

  return def;
}

int variagles_set_from_line(KVS *variables, const char *line) {
  char *buffer = strdup(line);
  if (!buffer) {
    fputs("Memory exhausted.\n", stderr);
    return -1;
  }

  char *sep = strchr(buffer, '=');
  if (sep) *sep = 0;

  char *key = trim_spaces(buffer, false);
  int result;
  if (sep) sep = unescape_value(trim_spaces(sep + 1, true));
  if (sep && *sep) {
    result = variables_set(variables, key, sep, false);
  } else {
    result = variables_unset(variables, key);
  }

  free(buffer);
  return result;
}
